from collections import defaultdict

from snapstraction.topology import _merge_topo_responses


def render_as_dot_digraph(topo, colorize_lq=False):
    """Yield a topology in DOT digraph format"""

    nodes = set()
    for (src, dst) in topo['topology'].keys():
        nodes.add(src)
        nodes.add(dst)

    nodes.discard(topo['root'])

    yield "digraph {"
    yield '    "{0}" [label="{0}\\n(root)",shape="doublecircle"];'.format(topo['root'])

    for node in sorted(nodes):
        yield '    "{}" [shape="circle"];'.format(node)

    for (src, dst) in sorted(topo['topology'].keys()):
        lq = topo['topology'][(src, dst)]
        if colorize_lq:
            lq_attrs = dot_attribs_to_colorize_lq(lq)
        else:
            lq_attrs = ""
        yield '    "{0}" -> "{1}" [label="{2}",weight="{2}"{3}];'.format(src, dst, lq, lq_attrs)
    yield "}"


def simplify_topology(topo):
    """Merge A->B and B->A pairs into single A-B edges"""

    simpler = defaultdict(list)

    for (src, dst) in topo['topology'].keys():
        new_key = tuple(sorted((src, dst)))
        simpler[new_key].append(topo['topology'][(src, dst)])

    merged = _merge_topo_responses(simpler)

    return {'root': topo['root'], 'topology': merged}


def render_as_dot_simple_graph(topo, colorize_lq=False):
    """Yield a topology in DOT graph format, simplified"""

    nodes = set()
    for (src, dst) in topo['topology'].keys():
        nodes.add(src)
        nodes.add(dst)

    nodes.discard(topo['root'])

    yield "graph {"
    yield '    "{0}" [label="{0}\\n(root)",shape="doublecircle"];'.format(topo['root'])

    for node in sorted(nodes):
        yield '    "{}" [shape="circle"];'.format(node)

    for (src, dst) in sorted(topo['topology'].keys()):
        lq = topo['topology'][(src, dst)]
        if colorize_lq:
            lq_attrs = dot_attribs_to_colorize_lq(lq)
        else:
            lq_attrs = ""
        yield '    "{0}" -- "{1}" [label="{2}",weight="{2}"{3}];'.format(src, dst, lq, lq_attrs)
    yield "}"


def dot_attribs_to_colorize_lq(lq, thresholds=[30, 80]):
    """Emits colorization attributes based on lq"""
    if len(thresholds) != 2:
        return ""
    if lq == 0:
        return ",color=black,penwidth=4.0"
    elif lq < thresholds[0]:
        return ",color=green,penwidth=4.0"
    elif lq < thresholds[1]:
        return ",color=orange,penwidth=2.0"
    else:
        return ",color=red,penwidth=1.0"
