"""SNAPtoolbelt Update Commands"""
import logging
import sys

from textwrap import dedent
from argparse import RawDescriptionHelpFormatter
from arghandler import ArgumentHandler

import pip

log = logging.getLogger(__name__)

PYPI_SERVER = "https://update.synapse-wireless.com/pypi"


def update(parser, context, args):
    """Update commands"""
    handler = ArgumentHandler(
        usage="snap [global options] update (check | install) [options]",
        description=dedent("""
        SNAPtoolbelt Update Commands:
          check: Check for updates
          install: Install updates
        """),
        formatter_class=RawDescriptionHelpFormatter,
    )

    handler.set_subcommands({
        'check': check_for_updates,
        'install': install_updates
    })

    handler.run(args, context_fxn=lambda x: context)


def check_for_updates(parser, context, args):
    """Ask pip to check for updated toolbelt & dependencies"""
    handler = ArgumentHandler(usage="snap update check [-h] [--pre]",
                              description="Check for updates to SNAPtoolbelt.")
    handler.add_argument("--pre",
                         action="store_true",
                         help="Allow pre-release (alpha) versions")

    args, rest = handler.parse_known_args(args)

    if rest:
        log.error("Unexpected args to update command: %r", rest)
        sys.exit(1)

    if args.pre:
        pip.main(['list', '--pre', '--outdated', '--extra-index-url',
                  PYPI_SERVER, 'snaptoolbelt'])
    else:
        pip.main(['list', '--outdated', '--extra-index-url', PYPI_SERVER,
                  'snaptoolbelt'])


def install_updates(parser, context, args):
    """Ask pip to upgrade toolbelt & dependencies"""
    handler = ArgumentHandler(usage="snap update install [-h] [--pre]",
                              description="Install updates to SNAPtoolbelt.")
    handler.add_argument("--pre",
                         action="store_true",
                         help="Allow pre-release (alpha) versions")

    args, rest = handler.parse_known_args(args)

    if rest:
        log.error("Unexpected args to update command: %r", rest)
        sys.exit(1)

    if args.pre:
        pip.main(['install', '--pre', '--upgrade', '--extra-index-url',
                  PYPI_SERVER, 'snaptoolbelt'])
    else:
        pip.main(['install', '--upgrade', '--extra-index-url', PYPI_SERVER,
                  'snaptoolbelt'])
